@@ -1,6 +1,4 @@
-/* $Id: main.c 2396 2007-01-17 17:43:59Z nick $
- * 
- * Copyright (c) 2003-2004 Roger Seguin <roger_seguin@msn.com>
+/*  Copyright (c) 2003-2004 Roger Seguin <roger_seguin@msn.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +32,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <memory.h>
+#include <math.h>
 #include <string.h>
 #include <inttypes.h>
 #include <string.h>
@@ -80,7 +79,7 @@ typedef enum monitor_bar_order_t {
 typedef struct param_t {
     /* Configurable parameters */
     char            acDevice[64];
-#if  !defined(__NetBSD__)
+#if  !defined(__NetBSD__) && !defined(__OpenBSD__)
     dev_t           st_rdev;
 #endif
     int             fTitleDisplayed;
@@ -151,15 +150,16 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
     const double    K = 1.0 * 1000 * 1000 * 1000 / 1024 / 1024;
     /* bytes/ns --> MB/s */
     double          arPerf[NMONITORS], arBusy[NMONITORS], *prData, *pr;
-    char            acToolTips[128];
+    char            acToolTips[256];
     int             status, i;
 
     if (!s_poToolTips)
 	s_poToolTips = gtk_tooltips_new ();
 
+    rbytes = wbytes = iRBusy_ns = iWBusy_ns = -1;
     memset (&oPerf, 0, sizeof (oPerf));
     oPerf.qlen = -1;
-#if defined (__NetBSD__)
+#if defined (__NetBSD__) || defined(__OpenBSD__)
     status = DevGetPerfData (poConf->acDevice, &oPerf);
 #else
     status = DevGetPerfData (&(poConf->st_rdev), &oPerf);
@@ -199,26 +199,26 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
 	}
     }
 
-    sprintf (acToolTips, "%s\n"
+    snprintf (acToolTips, sizeof(acToolTips), "%s\n"
 	     "----------------\n"
 	     "I/O    (MB/s)\n"
-	     "  Read :%3u\n"
-	     "  Write :%3u\n"
-	     "  Total :%3u\n"
+	     "  Read :%3.2f\n"
+	     "  Write :%3.2f\n"
+	     "  Total :%3.2f\n"
 	     "Busy time (%c)\n"
 	     "  Read : %3d\n"
 	     "  Write : %3d\n"
 	     "  Total : %3d",
 	     poConf->acTitle,
-	     (unsigned int) (arPerf[R_DATA] + 0.5),
-	     (unsigned int) (arPerf[W_DATA] + 0.5),
-	     (unsigned int) (arPerf[RW_DATA] + 0.5),
+	     arPerf[R_DATA],
+	     arPerf[W_DATA],
+	     arPerf[RW_DATA],
 	     '%',
 	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
-	     (int) (arBusy[R_DATA] + 0.5) : -1,
+	     (int) round(arBusy[R_DATA]) : -1,
 	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
-	     (int) (arBusy[W_DATA] + 0.5) : -1,
-	     (oPerf.qlen >= 0) ? (int) (arBusy[RW_DATA] + 0.5) : -1);
+	     (int) round(arBusy[W_DATA]) : -1,
+	     (oPerf.qlen >= 0) ? (int) round(arBusy[RW_DATA]) : -1);
     gtk_tooltips_set_tip (s_poToolTips, GTK_WIDGET (poMonitor->wEventBox),
 			  acToolTips, 0);
 
@@ -400,7 +400,7 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
     struct diskperf_t *poPlugin;
     struct param_t *poConf;
     struct monitor_t *poMonitor;
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 #endif
@@ -412,14 +412,14 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
 
     poPlugin->plugin = plugin;
     
-#if defined(__NetBSD__)
-    strcpy (poConf->acDevice, "wd0");
-    strcpy (poConf->acTitle, "wd0");
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+    strncpy (poConf->acDevice, "wd0", 64);
+    strncpy (poConf->acTitle, "wd0", 16);
 #else
-    strcpy (poConf->acDevice, "/dev/sda");
+    strncpy (poConf->acDevice, "/dev/sda", 64);
     status = stat (poConf->acDevice, &oStat);
     poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
-    strcpy (poConf->acTitle, "sda");
+    strncpy (poConf->acTitle, "sda", 16);
 #endif
 
     poConf->fTitleDisplayed = 1;
@@ -483,7 +483,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     Widget_t       *pw2ndBar = poPlugin->oMonitor.awProgressBar + 1;
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 #endif
@@ -500,7 +500,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     if ((value = xfce_rc_read_entry (rc, (CONF_DEVICE), NULL))) {
         memset (poConf->acDevice, 0, sizeof (poConf->acDevice));
         strncpy (poConf->acDevice, value, sizeof (poConf->acDevice) - 1);
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
         status = stat (poConf->acDevice, &oStat);
         poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
 #endif
@@ -551,6 +551,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     if ((value = xfce_rc_read_entry (rc, (CONF_READ_WRITE_COLOR), NULL))) {
         gdk_color_parse (value, poConf->aoColor + RW_DATA);
     }
+    SetMonitorBarColor (poPlugin);
 
     xfce_rc_close (rc);
 }				/* diskperf_read_config() */
@@ -597,17 +598,17 @@ static void diskperf_write_config (XfcePanelPlugin *plugin,
                              poConf->eMonitorBarOrder);
 
     poColor = poConf->aoColor + R_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_READ_COLOR, acBuffer);
 
     poColor = poConf->aoColor + W_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_WRITE_COLOR, acBuffer);
 
     poColor = poConf->aoColor + RW_DATA;
-    sprintf (acBuffer, acColorFormat,
+    snprintf (acBuffer, 16, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
     xfce_rc_write_entry (rc, CONF_READ_WRITE_COLOR, acBuffer);
 
@@ -623,7 +624,7 @@ static void SetDevice (Widget_t p_wTF, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcDevice = gtk_entry_get_text (GTK_ENTRY (p_wTF));
-#if !defined(__NetBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
     struct stat     oStat;
     int             status;
 
@@ -652,8 +653,6 @@ static void ToggleTitle (Widget_t p_w, void *p_pvPlugin)
 
     poConf->fTitleDisplayed =
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (p_w));
-    gtk_widget_set_sensitive (GTK_WIDGET (poGUI->wTF_Title),
-			      poConf->fTitleDisplayed);
     if (poConf->fTitleDisplayed)
 	gtk_widget_show (GTK_WIDGET (poMonitor->wTitle));
     else
@@ -755,15 +754,14 @@ static void SetXferRate (Widget_t p_wTF, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcXferRate = gtk_entry_get_text (GTK_ENTRY (p_wTF));
-    float           r;
-
-    poConf->iMaxXferMBperSec = atoi (pcXferRate);
 
     /* Make it a multiple of 5 MB/s */
-    r = (double) poConf->iMaxXferMBperSec / 5;
-    poConf->iMaxXferMBperSec = 5 * (int) (r + 0.5);
+    poConf->iMaxXferMBperSec = 5 * round((double) atoi(pcXferRate) / 5);
     if (poConf->iMaxXferMBperSec > 995)
 	poConf->iMaxXferMBperSec = 995;
+    else if (poConf->iMaxXferMBperSec < 5)
+	poConf->iMaxXferMBperSec = 5;
+    DBG("XferRate rounded to %dMb/s\n", poConf->iMaxXferMBperSec);
 }				/* SetXferRate() */
 
 	/**************************************************************/
@@ -778,12 +776,8 @@ static void SetPeriod (Widget_t p_wSc, void *p_pvPlugin)
     timerNeedsUpdate = 1;
     TRACE ("SetPeriod()\n");
     r = gtk_spin_button_get_value (GTK_SPIN_BUTTON (p_wSc));
-    poConf->iPeriod_ms = (r * 1000) + 0.5;	/* rounded */
-
-    /* Make it a multiple of 50 ms */
-    r = (double) poConf->iPeriod_ms / 50;
-    poConf->iPeriod_ms = 50 * (int) (r + 0.5);
-
+    poConf->iPeriod_ms = round(r * 1000);
+    DBG("Update period rounded to %dms\n", poConf->iPeriod_ms);
 }				/* SetPeriod() */
 
 	/**************************************************************/
@@ -950,8 +944,6 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (poGUI->wTB_Title),
 				  poConf->fTitleDisplayed);
-    gtk_widget_set_sensitive (GTK_WIDGET (poGUI->wTF_Title),
-			      poConf->fTitleDisplayed);
     g_signal_connect (GTK_WIDGET (poGUI->wTB_Title), "toggled",
 		      G_CALLBACK (ToggleTitle), poPlugin);
 
@@ -996,7 +988,7 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     g_signal_connect (GTK_WIDGET (poGUI->wTF_Title), "activate",
 		      G_CALLBACK (SetLabel), poPlugin);
 
-    sprintf (acBuffer, "%d", poConf->iMaxXferMBperSec);
+    snprintf (acBuffer, 16, "%d", poConf->iMaxXferMBperSec);
     gtk_entry_set_text (GTK_ENTRY (poGUI->wTF_MaxXfer), acBuffer);
     g_signal_connect (GTK_WIDGET (poGUI->wTF_MaxXfer), "activate",
 		      G_CALLBACK (SetXferRate), poPlugin);
